package core.taskgroups;

import core.entities.User;
import core.entities.timemanager.Taskgroup;
import core.repositories.UserRepository;
import core.repositories.timemanager.TaskgroupRepository;
import core.services.PermissionResult;
import core.services.ServiceExitCode;
import core.services.TaskgroupService;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.jdbc.SqlGroup;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Optional;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest
public class TaskgroupServiceTest {

    @Autowired
    private TaskgroupRepository taskgroupRepository;
    @Autowired
    private TaskgroupService taskgroupService;

    private static final String username = "Testuser1";
    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void getTaskgroupByIDAndUsername() {
        //Situation 1: correct taskgroup and username
        assertFalse(taskgroupRepository.findById(1L).isEmpty());

        PermissionResult<Taskgroup> permissionResult = taskgroupService.getTaskgroupByIDAndUsername(1L, username);
        assertThat(permissionResult).isNotNull();
        assertTrue(permissionResult.isHasPermissions());
        assertEquals(ServiceExitCode.OK, permissionResult.getExitCode());
        assertEquals(taskgroupRepository.findById(1L).get(), permissionResult.getResult());

        //Situation 2: invalid taskgroup
        PermissionResult<Taskgroup> invalid_group = taskgroupService.getTaskgroupByIDAndUsername(-1L, username);
        assertThat(invalid_group).isNotNull();
        assertEquals(ServiceExitCode.MISSING_ENTITY, invalid_group.getExitCode());
        assertFalse(invalid_group.isHasPermissions());

        //Situation 3: invalid user
        PermissionResult<Taskgroup> invalid_user = taskgroupService.getTaskgroupByIDAndUsername(1L, "Rotzbakke");
        assertThat(invalid_user).isNotNull();
        assertEquals(ServiceExitCode.OK, invalid_user.getExitCode());
        assertFalse(invalid_user.isHasPermissions());
    }

}
